#include "mat3x3.h"

NACB_BEGIN

void Mat3x3::print(const char * name) const
{
  printf("%s=[\n",name);
  for(int i=0;i<3;i++){
    for(int j=0;j<3;j++){
      printf("%6.5f ",data[3*i+j]); 
      }
    printf("\n");
  }
  printf("]\n");
}

Mat3x3 Mat3x3::inverse() const{
  Mat3x3 inv=Mat3x3::eye();
  Mat3x3 mat;
  mat.operator=(*this);
  
  for(int j=0;j<3;j++){
    int best=0;
    float best_val=0;
    for(int i=j;i<3;i++){
      if(fabs(mat(i,j))>best_val){
	best=i;
	best_val=fabs(mat(i,j));
      }
    }
    //swap the two rows
    if(j!=best){
      for(int jj=0;jj<3;jj++){
	double t=mat(j,jj);
	mat(j,jj)=mat(best,jj);
	mat(best,jj)=t;
	  
	t=inv(j,jj);
	inv(j,jj)=inv(best,jj);
	inv(best,jj)=t;
      }
    }
    //anihilate above and below
    for(int i=0;i<3;i++){
      if(j==i)continue;
      double sc=-mat(i,j)/mat(j,j);
	
      for(int k=0;k<3;k++){
	mat(i,k)+=sc*mat(j,k);
	inv(i,k)+=sc*inv(j,k);
      }
    }
    //make row one
    double sc=1.0/mat(j,j);
    for(int k=0;k<3;k++){
      mat(j,k)*=sc;
      inv(j,k)*=sc;
    }
  }
  return inv;
}

std::ostream & operator<<(std::ostream & os,const Mat3x3 & mat){
  mat.print(os);
  return os;
}

NACB_END
