#ifndef NACB_MAT3x3_H
#define NACB_MAT3x3_H

#include "nspace.h"
#include "vec3.h"

#include <sstream>

#ifdef __PY_PLUS_PLUS
#include <boost/python/tuple.hpp>
#include <boost/python/extract.hpp>
using boost::python::tuple;
using boost::python::extract;
#endif

NACB_BEGIN


class Mat3x3{
 public:
  double data[9];
  Mat3x3(){}

  void set(int i,double d){
    operator[](i)=d;
  }
  double get(int i,int j) const{return data[3*i+j];}
  double & operator()(int i,int j){return data[3*i+j];}
  const double & operator()(int i,int j) const {return data[3*i+j];}
  double & operator[](int ind){return data[ind];}
  const double & operator[](int ind) const {return data[ind];}

  template <class T>
    static Mat3x3 skewSym(const Vec3<T> & p){
    Mat3x3 ret;
    ret(0,0)= 0;
    ret(0,1)=-p.z;
    ret(0,2)= p.y;
    ret(1,0)= p.z;
    ret(1,1)= 0;
    ret(1,2)=-p.x;
    ret(2,0)=-p.y;
    ret(2,1)= p.x;
    ret(2,2)= 0;
    return ret;
  }

  template <class T2>
    Vec3<T2> operator*(const Vec3<T2> & p) const{
    return Vec3d(data[0]*p.x+data[1]*p.y+data[2]*p.z,
		 data[3]*p.x+data[4]*p.y+data[5]*p.z,
		 data[6]*p.x+data[7]*p.y+data[8]*p.z);
  }
  Mat3x3 operator+(const Mat3x3 & o)const{
    Mat3x3 m;
    for(int i=0;i<9;i++)
      m.data[i]=data[i]+o.data[i];
    return m;
  }
  Mat3x3 operator-(const Mat3x3 & o)const{
    Mat3x3 m;
    for(int i=0;i<9;i++)
      m.data[i]=data[i]-o.data[i];
    return m;
  }
  void operator+=(const Mat3x3 & o){
    for(int i=0;i<9;i++)
      data[i]+=o.data[i];
  }
  Mat3x3 operator*(const Mat3x3 & p) const{
    Mat3x3 res;
    for(int i=0;i<3;i++){
      for(int j=0;j<3;j++){
	double sum=0;
	for(int k=0;k<3;k++)
	  sum+=get(i,k)*p.get(k,j);
	res(i,j)=sum;
      }
    }
    return res;
  }
  Mat3x3 transpose() const{
    Mat3x3 mat;
    for(int i=0;i<3;i++){
      for(int j=0;j<3;j++){
	mat(i,j)=get(j,i);
      }
    }
    return mat;
  }
  static Mat3x3 diag(double a,double b,double c){
    Mat3x3 m;
    m.zero();
    for(int i=0;i<9;i++)
      m.data[i]=0;
    m.data[0]=a;
    m.data[4]=b;
    m.data[8]=c;
    return m;
  }
  static Mat3x3 eye(){
    Mat3x3 e;
    for(int i=0;i<9;i++){
      e.data[i]=0;
    }
    e.data[0]=e.data[4]=e.data[8]=1;
    return e;
  }
  static Mat3x3 zeros(){
    Mat3x3 m;
    for(int i=0;i<9;i++)
      m.data[i]=0;
    return m;
  }
  Mat3x3 inverse() const;

  void zero(){memset(data,0,sizeof(double)*9);}

  operator std::string () const{
    std::ostringstream os;
    print(os);
    return os.str();
  }
  void print(const char * name="") const;
  void print(std::ostream & os) const {
    for(int i=0;i<3;i++){
      for(int j=0;j<3;j++)
	os << get(i,j) << " ";
      os << "\n";
    }
  }

#ifdef __PY_PLUS_PLUS
  void python_set(const tuple & t,double d){
    operator()(extract<int>(t[0]),extract<int>(t[1]))=d;
  }
  double & operator[](const tuple & t){
    return operator()(extract<int>(t[0]),extract<int>(t[1]));
  }
#endif

};

std::ostream & operator<<(std::ostream & os,const Mat3x3 & mat);

NACB_END

#endif
