#ifndef NACB_VEC2_H
#define NACB_VEC2_H

#include <math.h>
#include <algorithm>
#include "nspace.h"

NACB_BEGIN

template <class T>
class Vec2{
 public:
#ifdef __PY_PLUS_PLUS
  T  x,y;
#else
  union{
    struct{
      T x,y;
    };
    T data[2];
  };
#endif
  Vec2(const T & xin=0,const T & yin=0):x(xin),y(yin){}
  template <class T2> 
    Vec2(const Vec2<T2> & o):x(o.x),y(o.y){}
  
  double dot(const Vec2 & o) const{
    return x*o.x+y*o.y;
  }
  double len() const{
    return sqrt(dot(*this));
  }
  double normalize(){
    double l=len();
    operator/=(l);
    return l;
  }
  Vec2 operator*(const double & d) const{
    return Vec2(x*d,y*d);
  }
  Vec2 operator+(const Vec2 & o) const{
    return Vec2(x+o.x,y+o.y);
  }
  Vec2 operator-(const Vec2 & o) const{
    return Vec2(x-o.x,y-o.y);
  }
  Vec2 operator/(const double & d)const{
    return Vec2(x/d,y/d);
  }
  Vec2 max(const Vec2 & o) const{
    return Vec2(std::max(x,o.x),std::max(y,o.y));
  }
  Vec2 min(const Vec2 & o) const{
    return Vec2(std::min(x,o.x),std::min(y,o.y));
  }
  void operator*=(const double & d){
    x*=d;
    y*=d;
  }
  void operator+=(const Vec2 & o){
    x+=o.x;
    y+=o.y;
  }
  void operator-=(const Vec2 & o){
    x-=o.x;
    y-=o.y;
  }
  void operator/=(const double & d){
    if(fabs(d)>1e-15){
      x/=d;
      y/=d;
    }
    else{
      x=y=0;
    }
  }
  bool operator==(const Vec2 & o) const{
    return (o.x==x) && (o.y==y);
  }
  template<class T2>
    operator Vec2<T2> () const{
    return Vec2<T2>(x,y);
  };
  T * getData(){
#ifdef __PY_PLUS_PLUS
    return &x;
#else
    return data;
#endif
  }
  T & operator[](int i){
    return getData()[i];
  }
#ifdef __PY_PLUS_PLUS
  void python_set(int i,T t){
    getData()[i]=t;
  }
#endif
};

typedef Vec2<float> Vec2f;
typedef Vec2<double> Vec2d;

#define VEC2_INSTANTIATIONS template class Vec2<double>; \
  template class Vec2<float>;   

#ifdef VEC2_INSTANTIATE
VEC2_INSTANTIATIONS
#endif 

NACB_END

#endif
